import {Injectable, OnDestroy} from '@angular/core'
import {Observable, Subject} from 'rxjs'
import {take, takeUntil} from 'rxjs/operators'
import {AppState} from '../../shared/app-state'
import {HttpClient} from '@angular/common/http'
import {environment} from '../../../../environments/environment'
import {ApiService} from '../../shared/service/api.service'
import {LoginDto, Permission} from '../../shared/model/account.model'
import {ErrorService} from '../../shared/service/error.service'
import {JwtService} from "./jwt.service";

@Injectable({
  providedIn: 'root'
})
export class AccountService implements OnDestroy {
  private destroy$ = new Subject<boolean>()

  constructor(
    private http: HttpClient,
    private api: ApiService,
    private appState: AppState,
    private jwtService: JwtService,
    private errorService: ErrorService
  ) {
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  loginAsGroupIfNotAuthenticated() {
    if (this.appState.isAuthenticated) return

    this.loginAsGroup()
  }

  loginAsUser(id: number, password: string): Observable<LoginDto> {
    return this.login(false, {id, password}, error => {
      if (error.status !== 403) return false

      this.errorService.handleError({status: error.status, type: 'invalid-credentials', obj: error})
      return true
    })
  }

  loginAsGroup(): Observable<LoginDto> {
    return this.login(true, {},
      error => error.status === 403)  // There is no group token, so do nothing
  }

  private login(isGroup: boolean, body: LoginBody, errorConsumer: (any) => boolean): Observable<LoginDto> {
    let url = `${environment.apiUrl}/account/login`
    if (isGroup) {
      url += '/group'
    }

    const login$ = this.http.post<LoginDto>(url, body, {withCredentials: true})
      .pipe(take(1), takeUntil(this.destroy$))

    login$.subscribe({
      next: result => this.appState.authenticateUser(result),
      error: error => {
        if (errorConsumer(error)) return;

        this.errorService.handleErrorResponse(error)
      }
    })

    return login$
  }

  logout() {
    this.appState.resetAuthenticatedUser()
    this.loginAsGroupIfNotAuthenticated()
  }

  hasPermission(permission: Permission): boolean {
    return this.appState.authenticatedUser && this.appState.authenticatedUser.permissions.indexOf(permission) >= 0
  }
}

interface LoginBody {
  id?: number
  password?: string
}
